@@ -24,7 +24,8 @@
 
 #include <config.h>
 
-#define  SPI_DEBUG
+#undef  SPI_DEBUG
+#undef  SPI_KEYEVENT_DEBUG
 
 #include <string.h>
 #include <ctype.h>
@@ -83,7 +84,7 @@ typedef struct {
 static void     spi_controller_register_with_devices          (SpiDEController           *controller);
 static gboolean spi_controller_update_key_grabs               (SpiDEController           *controller,
 							       Accessibility_DeviceEvent *recv);
-static void     spi_controller_register_device_listener       (SpiDEController           *controller,
+static gboolean spi_controller_register_device_listener       (SpiDEController           *controller,
 							       DEControllerListener      *l,
 							       CORBA_Environment         *ev);
 static void     spi_device_event_controller_forward_key_event (SpiDEController           *controller,
@@ -238,6 +239,9 @@ handle_keygrab (SpiDEController         *controller,
   if (key_listener->keys->_length == 0) /* special case means AnyKey/AllKeys */
     {
       grab_mask.key_val = AnyKey;
+#ifdef SPI_DEBUG
+      fprintf (stderr, "AnyKey grab!"); */
+#endif
       process_cb (controller, &grab_mask);
     }
   else
@@ -246,25 +250,33 @@ handle_keygrab (SpiDEController         *controller,
 
       for (i = 0; i < key_listener->keys->_length; ++i)
         {
-	  long int key_val = key_listener->keys->_buffer[i];
+	  Accessibility_KeyDefinition keydef = key_listener->keys->_buffer[i];	
+	  long int key_val = keydef.keysym;
 	  /* X Grabs require keycodes, not keysyms */
-	  if (key_val >= 0)
+	  if (keydef.keystring && keydef.keystring[0])
+	    {
+	      key_val = XStringToKeysym(keydef.keystring);		    
+	    }
+	  if (key_val > 0)
 	    {
 	      key_val = XKeysymToKeycode (spi_get_display (), (KeySym) key_val);
 	    }
+	  else
+	    {
+	      key_val = keydef.keycode;
+	    }
 	  grab_mask.key_val = key_val;
-
 	  process_cb (controller, &grab_mask);
 	}
     }
 }
 
-static void
+static gboolean
 spi_controller_register_global_keygrabs (SpiDEController         *controller,
 					 DEControllerKeyListener *key_listener)
 {
   handle_keygrab (controller, key_listener, _register_keygrab);
-  spi_controller_update_key_grabs (controller, NULL);
+  return spi_controller_update_key_grabs (controller, NULL);
 }
 
 static void
@@ -275,7 +287,7 @@ spi_controller_deregister_global_keygrabs (SpiDEController         *controller,
   spi_controller_update_key_grabs (controller, NULL);
 }
 
-static void
+static gboolean
 spi_controller_register_device_listener (SpiDEController      *controller,
 					 DEControllerListener *listener,
 					 CORBA_Environment    *ev)
@@ -290,12 +302,15 @@ spi_controller_register_device_listener (SpiDEController      *controller,
 						  key_listener);
       if (key_listener->mode->global)
         {
-	  spi_controller_register_global_keygrabs (controller, key_listener);	
+	  return spi_controller_register_global_keygrabs (controller, key_listener);	
 	}
+      else
+	      return TRUE;
       break;
     default:
       break;
   }
+  return FALSE; 
 }
 
 static GdkFilterReturn
@@ -346,6 +361,7 @@ spi_controller_register_with_devices (SpiDEController *controller)
   x_default_error_handler = XSetErrorHandler (_spi_controller_device_error_handler);
 }
 
+#define SPI_KEYEVENT_DEBUG
 static gboolean
 spi_key_set_contains_key (Accessibility_KeySet            *key_set,
 			  const Accessibility_DeviceEvent *key_event)
@@ -370,15 +386,21 @@ spi_key_set_contains_key (Accessibility_KeySet            *key_set,
   for (i = 0; i < len; ++i)
     {
 #ifdef SPI_KEYEVENT_DEBUG	    
-      g_print ("key_set[%d] = %d; key_event %d, code %d\n",
-	        i, (int) key_set->_buffer[i],
-	       (int) key_event->id, (int) key_event->hw_code); 
+      g_print ("key_set[%d] = %d; key_event %d, code %d, string %s\n",
+	        i, (int) key_set->_buffer[i].keycode,
+	       (int) key_event->id, (int) key_event->hw_code,
+	       key_event->event_string); 
 #endif
-      if (key_set->_buffer[i] == (CORBA_long) key_event->id)
+      if (key_set->_buffer[i].keysym == (CORBA_long) key_event->id)
         {
           return TRUE;
 	}
-      if (key_set->_buffer[i] == (CORBA_long) -key_event->hw_code)
+      if (key_set->_buffer[i].keycode == (CORBA_long) key_event->hw_code)
+        {
+          return TRUE;
+	}
+      if (key_event->event_string && key_event->event_string[0] &&
+	  !strcmp (key_set->_buffer[i].keystring, key_event->event_string))
         {
           return TRUE;
 	}
@@ -474,7 +496,7 @@ spi_notify_keylisteners (GList                          **key_listeners,
 #ifdef SPI_KEYEVENT_DEBUG
   if (!notify)
     {
-      g_print ("no match for listener %d\n", i);
+      g_print ("no match for event\n");
     }
 #endif
 
@@ -534,6 +556,9 @@ spi_keystroke_from_x_key_event (XKeyEvent *x_key_event)
         key_event.event_string = CORBA_string_dup ("space");
         break;
       case XK_Tab:
+#ifdef SPI_KEYEVENT_DEBUG
+	fprintf(stderr, "Tab\n");
+#endif
         key_event.event_string = CORBA_string_dup ("Tab");
 	break;
       case XK_BackSpace:
@@ -542,6 +567,69 @@ spi_keystroke_from_x_key_event (XKeyEvent *x_key_event)
       case XK_Return:
         key_event.event_string = CORBA_string_dup ("Return");
 	break;
+      case XK_Home:
+        key_event.event_string = CORBA_string_dup ("Home");
+	break;
+      case XK_Page_Down:
+        key_event.event_string = CORBA_string_dup ("Page_Down");
+	break;
+      case XK_Page_Up:
+        key_event.event_string = CORBA_string_dup ("Page_Up");
+	break;
+      case XK_F1:
+        key_event.event_string = CORBA_string_dup ("F1");
+	break;
+      case XK_F2:
+        key_event.event_string = CORBA_string_dup ("F2");
+	break;
+      case XK_F3:
+        key_event.event_string = CORBA_string_dup ("F3");
+	break;
+      case XK_F4:
+        key_event.event_string = CORBA_string_dup ("F4");
+	break;
+      case XK_F5:
+        key_event.event_string = CORBA_string_dup ("F5");
+	break;
+      case XK_F6:
+        key_event.event_string = CORBA_string_dup ("F6");
+	break;
+      case XK_F7:
+        key_event.event_string = CORBA_string_dup ("F7");
+	break;
+      case XK_F8:
+        key_event.event_string = CORBA_string_dup ("F8");
+	break;
+      case XK_F9:
+        key_event.event_string = CORBA_string_dup ("F9");
+	break;
+      case XK_F10:
+        key_event.event_string = CORBA_string_dup ("F10");
+	break;
+      case XK_F11:
+        key_event.event_string = CORBA_string_dup ("F11");
+	break;
+      case XK_F12:
+        key_event.event_string = CORBA_string_dup ("F12");
+	break;
+      case XK_End:
+        key_event.event_string = CORBA_string_dup ("End");
+	break;
+      case XK_Escape:
+        key_event.event_string = CORBA_string_dup ("Escape");
+	break;
+      case XK_Up:
+        key_event.event_string = CORBA_string_dup ("Up");
+	break;
+      case XK_Down:
+        key_event.event_string = CORBA_string_dup ("Down");
+	break;
+      case XK_Left:
+        key_event.event_string = CORBA_string_dup ("Left");
+	break;
+      case XK_Right:
+        key_event.event_string = CORBA_string_dup ("Right");
+	break;
       default:
         if (XLookupString (x_key_event, cbuf, cbuf_bytes, &keysym, NULL) > 0)
           {
@@ -688,7 +776,7 @@ spi_device_event_controller_object_finalize (GObject *object)
  * CORBA Accessibility::DEController::registerKeystrokeListener
  *     method implementation
  */
-static void
+static CORBA_boolean
 impl_register_keystroke_listener (PortableServer_Servant                  servant,
 				  const Accessibility_DeviceEventListener l,
 				  const Accessibility_KeySet             *keys,
@@ -705,8 +793,8 @@ impl_register_keystroke_listener (PortableServer_Servant                  servan
 	   (void *) l, (unsigned long) mask);
 #endif
   dec_listener = spi_dec_key_listener_new (l, keys, mask, type, mode, ev);
-  spi_controller_register_device_listener (
-    controller, (DEControllerListener *) dec_listener, ev);
+  return spi_controller_register_device_listener (
+	  controller, (DEControllerListener *) dec_listener, ev);
 }
 
 
